﻿Option Explicit On

Imports Excel = Microsoft.Office.Interop.Excel
Imports Microsoft.Office.Tools.Excel
Imports System.Data.OleDb
Imports System.Diagnostics

Module excel_sql_server_test

#Region "Description"

    ' The module contains examples of import/export/update operations for Excel and SQL Server.

    ' It shows three techniques:

    ' 1. Native Excel QueryTable object
    ' 2. Native Excel Range object and SqlCommand
    ' 3. VSTO ListObject

    ' Call the following procedures for test purposes:

    ' Sub TestImport_QueryTable(ByVal ws As Excel.Worksheet)
    ' Sub TestExport_QueryTable(ByVal ws As Excel.Worksheet)
    ' Sub TestRefresh_QueryTable(ByVal ws As Excel.Worksheet)

    ' Sub TestImport_SqlCommand(ByVal ws As Excel.Worksheet)
    ' Sub TestExport_SqlCommand(ByVal ws As Excel.Worksheet)
    ' Sub TestRefresh_SqlCommand(ByVal ws As Excel.Worksheet)

    ' Sub TestImport_ToolsListObject(ByVal ws As Excel.Worksheet)
    ' Sub TestExport_ToolsListObject(ByVal ws As Excel.Worksheet)
    ' Sub TestRefresh_ToolsListObject(ByVal ws As Excel.Worksheet)

    ' The examples works with a database hosted in Microsoft SQL Azure Database.
    ' You can modify the following test procedures for a new connection:

    ' Function TestConnectionString() As String
    ' Function TestCommandText() As String


    ' Take a look at the SaveToDB add-in at https://www.savetodb.com.
    ' The add-in allows saving data from Microsoft Excel to databases and web services.
    ' Possibly, the free SaveToDB Express edition already solves your tasks completely.
    ' The SaveToDB add-in has a lot of powerful features for free.
    ' SaveToDB Framework allows creating a complete database infrastructure for customizing Excel applications.
    ' SaveToDB SDK includes a lot Of examples.

#End Region

#Region "Test Connection String and CommandText"

    Function TestConnectionString() As String

        'TestConnectionString = GetConnectionString(".", "AzureDemo50", "", "")
        TestConnectionString = GetConnectionString("mssql.savetodb.com", "AzureDemo50", "excel_user", "ExSQL_#02")

    End Function

    Function TestCommandText() As String

        TestCommandText = "SELECT * FROM dbo04.ExcelTest"
        ' TestCommandText = "EXEC dbo04.uspExcelTest"

    End Function

#End Region

#Region "Test Procedures for Query Tables"

    Public Sub TestImport_QueryTable(ByVal ws As Excel.Worksheet)

        Dim obj As Object = ws.Application.InputBox(Prompt:="Select a cell",
            Default:=ws.Application.ActiveCell.Address, Type:=8)

        If Not IsReference(obj) Then            ' A user rejects the input
            Exit Sub
        End If

        Dim target As Excel.Range = obj

        If Not target.ListObject Is Nothing Then
            MsgBox("Select an empty cell to insert a data table")
            Exit Sub
        End If

        Dim connnectionString As String = TestConnectionString()
        Dim commandText As String = TestCommandText()

        Select Case CreateQueryTable(connnectionString, commandText, target)
            Case 0
            Case 1
                MsgBox(LastErrorMessage, MsgBoxStyle.Exclamation)
            Case Else
        End Select

    End Sub

    Sub TestExport_QueryTable(ByVal ws As Excel.Worksheet)

        On Error Resume Next

        Dim lo As Excel.ListObject = ws.Application.ActiveCell.ListObject

        If lo Is Nothing OrElse lo.QueryTable Is Nothing Then
            MsgBox("Select a connected ListObject table to export data")
            Exit Sub
        End If

        Dim connnectionString As String = TestConnectionString()

        Select Case ExportRangeToDatabase(lo.QueryTable.ResultRange, connnectionString,
                "dbo04.ExcelTestImport",
                "EXEC dbo04.uspImportExcel_Before",
                "EXEC dbo04.uspImportExcel_After")
            Case 0
            Case 1
                MsgBox(LastErrorMessage, MsgBoxStyle.Exclamation)
            Case Else
        End Select

        lo.QueryTable.Refresh(BackgroundQuery:=True)

    End Sub

    Sub TestRefresh_QueryTable(ByVal ws As Excel.Worksheet)

        On Error Resume Next

        Dim lo As Excel.ListObject = ws.Application.ActiveCell.ListObject

        If lo Is Nothing OrElse lo.QueryTable Is Nothing Then
            Exit Sub
        End If

        lo.QueryTable.Refresh(BackgroundQuery:=True)

    End Sub

#End Region

#Region "Test Procedures for use of SQL Commands"

    ''' <summary>
    ''' The common function for TestImport and TestRefresh.
    ''' </summary>
    ''' <param name="target">The target range or cell to insert the data.</param>
    ''' <remarks>
    ''' The refresh is the same as import. So the common procedure is used.
    ''' </remarks>
    Private Sub TestImport_SqlCommand(target As Excel.Range)

        Dim connnectionString As String = TestConnectionString()
        Dim commandText As String = TestCommandText()

        Select Case ImportSQLtoRange(connnectionString, commandText, target)
            Case 0
            Case 1
                MsgBox(LastErrorMessage, MsgBoxStyle.Exclamation)
            Case Else
        End Select

    End Sub

    Sub TestImport_SqlCommand(ByVal ws As Excel.Worksheet)

        Dim obj As Object = ws.Application.InputBox(Prompt:="Select a cell",
            Default:=ws.Application.ActiveCell.Address, Type:=8)

        If Not IsReference(obj) Then            ' A user rejects the input
            Exit Sub
        End If

        Dim target As Excel.Range = obj

        If Not target.ListObject Is Nothing Then
            MsgBox("Select an empty cell to insert a data table")
            Exit Sub
        End If

        TestImport_SqlCommand(target)

    End Sub

    Sub TestExport_SqlCommand(ByVal ws As Excel.Worksheet)

        On Error Resume Next

        Dim listObject As Excel.ListObject = ws.Application.ActiveCell.ListObject

        If listObject Is Nothing Then
            MsgBox("Select a DataTable to export data")
            Exit Sub
        End If

        Dim connnectionString As String = TestConnectionString()

        Select Case ExportRangeToDatabase(listObject.Range, connnectionString,
                "dbo04.ExcelTestImport",
                "EXEC dbo04.uspImportExcel_Before",
                "EXEC dbo04.uspImportExcel_After")
            Case 0
            Case 1
                MsgBox(LastErrorMessage, MsgBoxStyle.Exclamation)
            Case Else
        End Select

        TestRefresh_SqlCommand(ws)

    End Sub

    Sub TestRefresh_SqlCommand(ByVal ws As Excel.Worksheet)

        On Error Resume Next

        Dim listObject As Excel.ListObject = ws.Application.ActiveCell.ListObject

        If listObject Is Nothing Then
            MsgBox("Select a DataTable to refresh data")
            Exit Sub
        End If

        ' Refresh deletes the current ListObject and inserts a new one

        Dim target As Excel.Range = listObject.Range.Cells(1, 1)

        listObject.Range.Clear()

        TestImport_SqlCommand(target)

    End Sub

#End Region

#Region "Test Procedures for Tools ListObjects"

    ''' <summary>
    ''' The function returns Tools Worksheet for Excel Worksheet.
    ''' </summary>
    ''' <param name="ws">The Excel worksheet</param>
    ''' <returns>The Tools Worksheet</returns>
    ''' <remarks>
    ''' The function returns Nothing if the Excel worksheet is not a Tools worksheet.
    ''' </remarks>
    Function FindToolsWorksheet(ByVal ws As Excel.Worksheet) As WorksheetBase

        ' This is the simplest way for test purposes.
        ' VSTO workbook has no collection of VSTO worksheets

        Select Case ws.Name
            Case Globals.Sheet1.Name
                Return Globals.Sheet1
            Case Else
                Return Nothing
        End Select

    End Function

    Sub TestImport_ToolsListObject(ByVal ws As Excel.Worksheet)

        Dim sh As WorksheetBase = FindToolsWorksheet(ws)

        If sh Is Nothing Then
            MsgBox("Select a Tools Worksheet")
            Exit Sub
        End If

        Dim obj As Object = ws.Application.InputBox(Prompt:="Select a cell",
            Default:=ws.Application.ActiveCell.Address, Type:=8)

        If Not IsReference(obj) Then            ' A user rejects the input
            Exit Sub
        End If

        Dim target As Excel.Range = obj

        If Not target.ListObject Is Nothing Then
            MsgBox("Select an empty cell to insert a data table")
            Exit Sub
        End If

        Dim connectionString As String = TestConnectionString()
        Dim commandText As String = TestCommandText()

        Select Case ImportToolsListObject(connectionString, commandText, sh, target)
            Case 0
            Case 1
                MsgBox(LastErrorMessage, MsgBoxStyle.Exclamation)
            Case Else
        End Select

    End Sub

    Sub TestExport_ToolsListObject(ByVal ws As Excel.Worksheet)

        Dim sh As WorksheetBase = FindToolsWorksheet(ws)

        If sh Is Nothing Then
            MsgBox("Select a Tools Worksheet")
            Exit Sub
        End If

        Dim listObject As Excel.ListObject = ws.Application.ActiveCell.ListObject

        If listObject Is Nothing Then
            MsgBox("Select a ListObject to export data")
            Exit Sub
        ElseIf sh.Controls.IndexOf(listObject.DisplayName) < 0 Then
            MsgBox("Select a Tools ListObject to export data")
            Exit Sub
        End If

        Dim connectionString As String = TestConnectionString()

        Dim toolsListObject As Microsoft.Office.Tools.Excel.ListObject = sh.Controls(listObject.DisplayName)

        Select Case ExportToolsListObject(connectionString, toolsListObject)
            Case 0
            Case 1
                MsgBox(LastErrorMessage, MsgBoxStyle.Exclamation)
            Case Else
        End Select

    End Sub

    Sub TestRefresh_ToolsListObject(ByVal ws As Excel.Worksheet)

        Dim sh As WorksheetBase = FindToolsWorksheet(ws)

        If sh Is Nothing Then
            MsgBox("Select a Tools Worksheet")
            Exit Sub
        End If

        Dim listObject As Excel.ListObject = ws.Application.ActiveCell.ListObject

        If listObject Is Nothing Then
            MsgBox("Select a ListObject to refresh data")
            Exit Sub
        ElseIf sh.Controls.IndexOf(listObject.DisplayName) < 0 Then
            MsgBox("Select a Tools ListObject to refresh data")
            Exit Sub
        End If

        Dim connectionString As String = TestConnectionString()

        Dim toolsListObject As Microsoft.Office.Tools.Excel.ListObject = sh.Controls(listObject.DisplayName)

        Select Case RefreshToolsListObject(connectionString, toolsListObject)
            Case 0
            Case 1
                MsgBox(LastErrorMessage, MsgBoxStyle.Exclamation)
            Case Else
        End Select

    End Sub

#End Region

End Module
